@@ -17,33 +17,117 @@
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "riff.h"
-#include "avi.h"
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+// C includes
+
+#include <stdio.h>
 #include <fcntl.h>
 #include <unistd.h>
 
+// C++ includes
+
 #include <string>
 #include <iostream>
 #include <strstream>
 #include <iomanip>
 
+using std::cout;
+using std::hex;
+using std::dec;
+using std::setw;
+using std::setfill;
+using std::endl;
+
+// local includes
+
+#include "error.h"
+#include "riff.h"
+#include "avi.h"
+
 #define PADDING_SIZE (512)
 #define IX00_INDEX_SIZE (4028)
 
 static char g_zeroes[PADDING_SIZE];
 
-AVIFile::AVIFile() : RIFFFile(), index_type( -1), current_ix00( -1), file_list( -1), riff_list( -1), hdrl_list( -1), avih_chunk( -1), movi_list( -1), indx_chunk( -1), ix00_chunk( -1), idx1_chunk( -1), junk_chunk( -1)
-{}
+AVIFile::AVIFile() : RIFFFile(), index_type( -1), current_ix00( -1), file_list( -1), riff_list( -1),
+        hdrl_list( -1), avih_chunk( -1), movi_list( -1), indx_chunk( -1), ix00_chunk( -1),
+        idx1_chunk( -1), junk_chunk( -1)
+{
+    // cerr << "0x" << hex << (long)this << dec << " AVIFile::AVIFile() : RIFFFile(), ..." << endl;
+
+    indx = new AVISuperIndex;
+    memset(indx, 0, sizeof(AVISuperIndex));
+    ix00 = new AVIStdIndex;
+    memset(ix00, 0, sizeof(AVIStdIndex));
+    idx1 = new AVISimpleIndex;
+    memset(idx1, 0, sizeof(AVISimpleIndex));
+}
 
 
+AVIFile::AVIFile(const AVIFile& avi) : RIFFFile(avi)
+{
+    // cerr << "0x" << hex << (long)this << dec << " 0x" << hex << (long)&avi << dec << " AVIFile::AVIFile(const AVIFile& avi) : RIFFFile(avi)" << endl;
 
-AVIFile::AVIFile(const char *s) : RIFFFile(s), index_type( -1), current_ix00( -1), file_list( -1), riff_list( -1), hdrl_list( -1), avih_chunk( -1), movi_list( -1), indx_chunk( -1), ix00_chunk( -1), idx1_chunk( -1), junk_chunk( -1)
-{}
+    mainHdr = avi.mainHdr;
+    indx = new AVISuperIndex;
+    *indx = *avi.indx;
+    ix00 = new AVIStdIndex;
+    *ix00 = *avi.ix00;
+    idx1 = new AVISimpleIndex;
+    *idx1 = *avi.idx1;
 
+    index_type = avi.index_type;
+    current_ix00 = avi.current_ix00;
+    file_list = avi.file_list;
+    riff_list = avi.riff_list;
+    hdrl_list = avi.hdrl_list;
+    avih_chunk = avi.avih_chunk;
+    movi_list = avi.movi_list;
+    indx_chunk = avi.indx_chunk;
+    ix00_chunk = avi.ix00_chunk;
+    idx1_chunk = avi.idx1_chunk;
+    junk_chunk = avi.junk_chunk;
+}
+
+
+AVIFile& AVIFile::operator=(const AVIFile& avi)
+{
+    // cerr << "0x" << hex << (long)this << dec << " 0x" << hex << (long)&avi << dec << " AVIFile& AVIFile::operator=(const AVIFile& avi)" << endl;
+
+    if (this != &avi) {
+        RIFFFile::operator=(avi);
+        mainHdr = avi.mainHdr;
+        *indx = *avi.indx;
+        *ix00 = *avi.ix00;
+        *idx1 = *avi.idx1;
+
+        index_type = avi.index_type;
+        current_ix00 = avi.current_ix00;
+        file_list = avi.file_list;
+        riff_list = avi.riff_list;
+        hdrl_list = avi.hdrl_list;
+        avih_chunk = avi.avih_chunk;
+        movi_list = avi.movi_list;
+        indx_chunk = avi.indx_chunk;
+        ix00_chunk = avi.ix00_chunk;
+        idx1_chunk = avi.idx1_chunk;
+        junk_chunk = avi.junk_chunk;
+    }
+    return *this;
+}
 
 
 AVIFile::~AVIFile()
-{}
+{
+    // cerr << "0x" << hex << (long)this << dec << " AVIFile::~AVIFile()" << endl;
+
+    delete indx;
+    delete ix00;
+    delete idx1;
+}
 
 /** Initialize the AVI structure to its initial state, either for PAL or NTSC format
  
@@ -72,7 +156,7 @@ void AVIFile::Init(int format, int sampleFrequency)
         mainHdr.dwSuggestedBufferSize = 120008;
         break;
 
-    default:                 /* no default allowed */
+    default: /* no default allowed */
         assert(0);
         break;
     }
@@ -94,18 +178,18 @@ void AVIFile::Init(int format, int sampleFrequency)
 
     /* Initialize the indx chunk */
 
-    indx.wLongsPerEntry = 4;
-    indx.bIndexSubType = 0;
-    indx.bIndexType = AVI_INDEX_OF_INDEXES;
-    indx.nEntriesInUse = 0;
-    // indx.dwChunkId = make_fourcc("00__");
-    indx.dwReserved[0] = 0;
-    indx.dwReserved[1] = 0;
-    indx.dwReserved[2] = 0;
+    indx->wLongsPerEntry = 4;
+    indx->bIndexSubType = 0;
+    indx->bIndexType = AVI_INDEX_OF_INDEXES;
+    indx->nEntriesInUse = 0;
+    // indx->dwChunkId = make_fourcc("00__");
+    indx->dwReserved[0] = 0;
+    indx->dwReserved[1] = 0;
+    indx->dwReserved[2] = 0;
     for (i = 0; i < 2014; ++i) {
-        indx.aIndex[i].qwOffset = 0;
-        indx.aIndex[i].dwSize = 0;
-        indx.aIndex[i].dwDuration = 0;
+        indx->aIndex[i].qwOffset = 0;
+        indx->aIndex[i].dwSize = 0;
+        indx->aIndex[i].dwDuration = 0;
     }
 
     /* The ix00 chunk will be added dynamically in avi_write_frame
@@ -128,27 +212,27 @@ void AVIFile::Init(int format, int sampleFrequency)
     \return 0 if the frame could be found, -1 otherwise
 */
 
-int AVIFile::GetDVFrameInfo(int &offset, int &size, int frameNum)
+int AVIFile::GetDVFrameInfo(off_t &offset, int &size, int frameNum)
 {
     switch (index_type) {
     case 1:
 
-	/* find relevant index in indx */
+        /* find relevant index in indx */
 
-	int i;	
+        int i;
 
-	for (i = 0; frameNum >= indx.aIndex[i].dwDuration; frameNum -= indx.aIndex[i].dwDuration)
-		;
+        for (i = 0; frameNum >= indx->aIndex[i].dwDuration; frameNum -= indx->aIndex[i].dwDuration, ++i)
+            ;
 
-	if (i != current_ix00) {
-	    lseek(fd, indx.aIndex[i].qwOffset + 8, SEEK_SET);
-    	    read(fd, &ix00, indx.aIndex[i].dwSize - 8);
-	    current_ix00 = i;
-	}
+        if (i != current_ix00) {
+            fail_if(lseek(fd, indx->aIndex[i].qwOffset + RIFF_HEADERSIZE, SEEK_SET) == (off_t)-1);
+            fail_neg(read(fd, ix00, indx->aIndex[i].dwSize - RIFF_HEADERSIZE));
+            current_ix00 = i;
+        }
 
-        if (frameNum < ix00.nEntriesInUse) {
-            offset = ix00.qwBaseOffset + ix00.aIndex[frameNum].dwOffset;
-            size = ix00.aIndex[frameNum].dwSize;
+        if (frameNum < ix00->nEntriesInUse) {
+            offset = ix00->qwBaseOffset + ix00->aIndex[frameNum].dwOffset;
+            size = ix00->aIndex[frameNum].dwSize;
             return 0;
         } else
             return -1;
@@ -158,8 +242,8 @@ int AVIFile::GetDVFrameInfo(int &offset, int &size, int frameNum)
         int index = -1;
         int frameNumIndex = 0;
         FOURCC chunkId = make_fourcc("00dc");
-        for (int i = 0; i < idx1.nEntriesInUse; ++i) {
-            if (idx1.aIndex[i].dwChunkId == chunkId) {
+        for (int i = 0; i < idx1->nEntriesInUse; ++i) {
+            if (idx1->aIndex[i].dwChunkId == chunkId) {
                 if (frameNumIndex == frameNum) {
                     index = i;
                     break;
@@ -168,8 +252,14 @@ int AVIFile::GetDVFrameInfo(int &offset, int &size, int frameNum)
             }
         }
         if (index != -1) {
-            offset = idx1.aIndex[index].dwOffset + 8;
-            size = idx1.aIndex[index].dwSize;
+            // compatibility check for broken dvgrab dv2 format
+            if (idx1->aIndex[0].dwOffset > GetDirectoryEntry(movi_list).offset) {
+                offset = idx1->aIndex[index].dwOffset + RIFF_HEADERSIZE;
+            } else {
+                // new, correct dv2 format
+                offset = idx1->aIndex[index].dwOffset + RIFF_HEADERSIZE + GetDirectoryEntry(movi_list).offset;
+            }
+            size = idx1->aIndex[index].dwSize;
             return 0;
         } else
             return -1;
@@ -190,13 +280,13 @@ int AVIFile::GetDVFrameInfo(int &offset, int &size, int frameNum)
 
 int AVIFile::GetDVFrame(Frame &frame, int frameNum)
 {
-    int	offset;
+    off_t	offset; // should be off_t
     int	size;
 
     if (GetDVFrameInfo(offset, size, frameNum) != 0)
         return -1;
-    lseek(fd, offset, SEEK_SET);
-    read(fd, frame.data, size);
+    fail_if(lseek(fd, offset, SEEK_SET) == (off_t)-1);
+    fail_neg(read(fd, frame.data, size));
 
     return 0;
 }
@@ -208,14 +298,13 @@ int AVIFile::GetTotalFrames() const
 }
 
 
-
 /** prints out a directory entry in text form
-
+ 
     Every subclass of RIFFFile is supposed to override this function
     and to implement it for the entry types it knows about. For all
     other entry types it should call its parent::PrintDirectoryData.
-
-    \todo user 64 bit routines
+ 
+    \todo use 64 bit routines
     \param entry the entry to print
 */
 
@@ -226,8 +315,8 @@ void AVIFile::PrintDirectoryEntryData(const RIFFDirEntry &entry) const
         int             i;
         MainAVIHeader   main_avi_header;
 
-        lseek(fd, entry.offset, SEEK_SET);
-        read(fd, &main_avi_header, sizeof(MainAVIHeader));
+        fail_if(lseek(fd, entry.offset, SEEK_SET) == (off_t)-1);
+        fail_neg(read(fd, &main_avi_header, sizeof(MainAVIHeader)));
 
         cout << "    dwMicroSecPerFrame:    " << (int)main_avi_header.dwMicroSecPerFrame << endl
         << "    dwMaxBytesPerSec:      " << (int)main_avi_header.dwMaxBytesPerSec << endl
@@ -246,8 +335,8 @@ void AVIFile::PrintDirectoryEntryData(const RIFFDirEntry &entry) const
 
         AVIStreamHeader	avi_stream_header;
 
-        lseek(fd, entry.offset, SEEK_SET);
-        read(fd, &avi_stream_header, sizeof(AVIStreamHeader));
+        fail_if(lseek(fd, entry.offset, SEEK_SET) == (off_t)-1);
+        fail_neg(read(fd, &avi_stream_header, sizeof(AVIStreamHeader)));
 
         cout << "    fccType:         '"
         << (char)((avi_stream_header.fccType >> 0) & 0x000000ff)
@@ -276,10 +365,11 @@ void AVIFile::PrintDirectoryEntryData(const RIFFDirEntry &entry) const
         int             i;
         AVISuperIndex   avi_super_index;
 
-        lseek(fd, entry.offset, SEEK_SET);
-        read(fd, &avi_super_index, sizeof(AVISuperIndex));
+        fail_if(lseek(fd, entry.offset, SEEK_SET) == (off_t)-1);
+        fail_neg(read(fd, &avi_super_index, sizeof(AVISuperIndex)));
 
-        cout << "    wLongsPerEntry: " <<  (int)avi_super_index.wLongsPerEntry << endl
+        cout << "    wLongsPerEntry: " <<  (int)avi_super_index.wLongsPerEntry
+        << endl
         << "    bIndexSubType:  " <<  (int)avi_super_index.bIndexSubType << endl
         << "    bIndexType:     " <<  (int)avi_super_index.bIndexType << endl
         << "    nEntriesInUse:  " <<  (int)avi_super_index.nEntriesInUse << endl
@@ -293,7 +383,7 @@ void AVIFile::PrintDirectoryEntryData(const RIFFDirEntry &entry) const
         << "    dwReserved[1]:  " <<  (int)avi_super_index.dwReserved[1] << endl
         << "    dwReserved[2]:  " <<  (int)avi_super_index.dwReserved[2] << endl;
         for (i = 0; i < avi_super_index.nEntriesInUse; ++i) {
-            cout << ' ' << setw(4) << setfill(' ') << i 
+            cout << ' ' << setw(4) << setfill(' ') << i
             << ": qwOffset    : 0x" << setw(12) << setfill('0') << hex << avi_super_index.aIndex[i].qwOffset << endl
             << "       dwSize      : 0x" << setw(8) << avi_super_index.aIndex[i].dwSize << endl
             << "       dwDuration  : " << dec << avi_super_index.aIndex[i].dwDuration << endl;
@@ -308,10 +398,11 @@ void AVIFile::PrintDirectoryEntryData(const RIFFDirEntry &entry) const
         int             i;
         AVIStdIndex     avi_std_index;
 
-        lseek(fd, entry.offset, SEEK_SET);
-        read(fd, &avi_std_index, sizeof(AVIStdIndex));
+        fail_if(lseek(fd, entry.offset, SEEK_SET) == (off_t)-1);
+        fail_neg(read(fd, &avi_std_index, sizeof(AVIStdIndex)));
 
-        cout << "    wLongsPerEntry: " << (int)avi_std_index.wLongsPerEntry << endl
+        cout << "    wLongsPerEntry: " << (int)avi_std_index.wLongsPerEntry
+        << endl
         << "    bIndexSubType:  " << (int)avi_std_index.bIndexSubType << endl
         << "    bIndexType:     " << (int)avi_std_index.bIndexType << endl
         << "    nEntriesInUse:  " << (int)avi_std_index.nEntriesInUse << endl
@@ -324,7 +415,7 @@ void AVIFile::PrintDirectoryEntryData(const RIFFDirEntry &entry) const
         << "    qwBaseOffset:   0x" << setw(12) << hex << avi_std_index.qwBaseOffset << endl
         << "    dwReserved:     " << dec << (int)avi_std_index.dwReserved << endl;
         for (i = 0; i < avi_std_index.nEntriesInUse; ++i) {
-            cout << ' ' << setw(4) << setfill(' ') << i 
+            cout << ' ' << setw(4) << setfill(' ') << i
             << ": dwOffset    : 0x" << setw(8) << setfill('0') << hex << avi_std_index.aIndex[i].dwOffset
             << " (0x" << avi_std_index.qwBaseOffset + avi_std_index.aIndex[i].dwOffset << ')' << endl
             << "       dwSize      : 0x" << setw(8) << avi_std_index.aIndex[i].dwSize << dec << endl;
@@ -336,8 +427,8 @@ void AVIFile::PrintDirectoryEntryData(const RIFFDirEntry &entry) const
         int numEntries = entry.length / sizeof(int) / 4;
         int *idx1 = new int[numEntries * 4];
 
-        lseek(fd, entry.offset, SEEK_SET);
-        read(fd, idx1, entry.length);
+        fail_if(lseek(fd, entry.offset, SEEK_SET) == (off_t)-1);
+        fail_neg(read(fd, idx1, entry.length));
 
         for (i = 0; i < numEntries; ++i) {
 
@@ -357,50 +448,60 @@ void AVIFile::PrintDirectoryEntryData(const RIFFDirEntry &entry) const
 }
 
 
+/** If this is not a movi list, read its contents
+ 
+*/
+
 void AVIFile::ParseList(int parent)
 {
-    FOURCC type;
-    FOURCC name;
-    int length;
-    int pos;
-    int list;
-    int listEnd;
+    FOURCC      type;
+    FOURCC      name;
+    int         length;
+    int         list;
+    off_t       pos;
+    off_t       listEnd;
 
     /* Read in the chunk header (type and length). */
-
     fail_neg(read(fd, &type, sizeof(type)));
     fail_neg(read(fd, &length, sizeof(length)));
-    if (length & 1) length++;
+    if (length & 1)
+        length++;
 
     /* The contents of the list starts here. Obtain its offset. The list
        name (4 bytes) is already part of the contents). */
 
-    fail_neg(pos = lseek(fd, 0, SEEK_CUR));
+    pos = lseek(fd, 0, SEEK_CUR);
+    fail_if(pos == (off_t)-1);
     fail_neg(read(fd, &name, sizeof(name)));
 
+    /* if we encounter a movi list, do not read it. It takes too much time
+       and we don't need it anyway. */
+
     if (name != make_fourcc("movi")) {
 
-    /* Add an entry for this list. */
+        /* Add an entry for this list. */
+        list = AddDirectoryEntry(type, name, sizeof(name), parent);
 
-    list = AddDirectoryEntry(type, name, sizeof(name), parent);
+        /* Read in any chunks contained in this list. This list is the
+           parent for all chunks it contains. */
 
-    /* Read in any chunks contained in this list. This list is the
-       parent for all chunks it contains. */
-
-    listEnd = pos + length;
-    while (pos < listEnd) {
-        ParseChunk(list);
-        fail_neg(pos = lseek(fd, 0, SEEK_CUR));
-    }
+        listEnd = pos + length;
+        while (pos < listEnd) {
+            ParseChunk(list);
+            pos = lseek(fd, 0, SEEK_CUR);
+            fail_if(pos == (off_t)-1);
+        }
     } else {
-    /* Add an entry for this list. */
+        /* Add an entry for this list. */
 
-    list = AddDirectoryEntry(type, name, length, parent);
+        movi_list = AddDirectoryEntry(type, name, length, parent);
 
-        fail_neg(pos = lseek(fd, length - 4, SEEK_CUR));
-    }  
+        pos = lseek(fd, length - 4, SEEK_CUR);
+        fail_if(pos == (off_t)-1);
+    }
 }
 
+
 void AVIFile::ParseRIFF()
 {
     RIFFFile::ParseRIFF();
@@ -415,26 +516,28 @@ void AVIFile::ReadIndex()
 {
     indx_chunk = FindDirectoryEntry(make_fourcc("indx"));
     if (indx_chunk != -1) {
-        ReadChunk(indx_chunk, (void*)&indx);
+        ReadChunk(indx_chunk, (void*)indx);
         index_type = 1;
+        return;
     }
     idx1_chunk = FindDirectoryEntry(make_fourcc("idx1"));
     if (idx1_chunk != -1) {
-        ReadChunk(idx1_chunk, (void*)&idx1);
-        idx1.nEntriesInUse = GetDirectoryEntry(idx1_chunk).length / 16;
+        ReadChunk(idx1_chunk, (void*)idx1);
+        idx1->nEntriesInUse = GetDirectoryEntry(idx1_chunk).length / 16;
         index_type = 2;
+        return;
     }
 }
 
 
 void AVIFile::FlushIX00Index()
 {
-    FOURCC type;
-    FOURCC name;
-    int length;
-    int offset;
-    int parent;
-    int i;
+    FOURCC      type;
+    FOURCC      name;
+    off_t       length;
+    off_t       offset;
+    int         parent;
+    int         i;
 
     /* Write out the old index. When this function is
        entered for the first time, there is no index to
@@ -443,7 +546,7 @@ void AVIFile::FlushIX00Index()
        position. */
 
     if (ix00_chunk != -1)
-        WriteChunk(ix00_chunk, &ix00);
+        WriteChunk(ix00_chunk, ix00);
 
     /* make a new ix00 chunk. */
 
@@ -455,51 +558,51 @@ void AVIFile::FlushIX00Index()
        offset to the next free location in the file
        there. */
 
-    ix00.wLongsPerEntry = 2;
-    ix00.bIndexSubType = 0;
-    ix00.bIndexType = AVI_INDEX_OF_CHUNKS;
-    ix00.nEntriesInUse = 0;
-    ix00.dwChunkId = indx.dwChunkId;
-    ix00.qwBaseOffset = offset + length;
-    ix00.dwReserved = 0;
+    ix00->wLongsPerEntry = 2;
+    ix00->bIndexSubType = 0;
+    ix00->bIndexType = AVI_INDEX_OF_CHUNKS;
+    ix00->nEntriesInUse = 0;
+    ix00->dwChunkId = indx->dwChunkId;
+    ix00->qwBaseOffset = offset + length;
+    ix00->dwReserved = 0;
 
     for (i = 0; i < IX00_INDEX_SIZE; ++i) {
-        ix00.aIndex[i].dwOffset = 0;
-        ix00.aIndex[i].dwSize = 0;
+        ix00->aIndex[i].dwOffset = 0;
+        ix00->aIndex[i].dwSize = 0;
     }
 
     /* add a reference to this new index in our super
                      index. */
 
-    i = indx.nEntriesInUse++;
-    indx.aIndex[i].qwOffset = offset - 8;
-    indx.aIndex[i].dwSize = length + 8;
-    indx.aIndex[i].dwDuration = 0;
+    i = indx->nEntriesInUse++;
+    indx->aIndex[i].qwOffset = offset - RIFF_HEADERSIZE;
+    indx->aIndex[i].dwSize = length + RIFF_HEADERSIZE;
+    indx->aIndex[i].dwDuration = 0;
 }
 
 
 void AVIFile::UpdateIndex(int frame_chunk)
 {
-    FOURCC type;
-    FOURCC name;
-    int length;
-    int offset;
-    int parent;
-    int i;
+    FOURCC      type;
+    FOURCC      name;
+    off_t       length;
+    off_t       offset;
+    int         parent;
+    int         i;
 
     /* update the appropiate entry in the super index. It reflects
        the number of frames in the referenced index. */
 
-    i = indx.nEntriesInUse - 1;
-    ++indx.aIndex[i].dwDuration;
+    i = indx->nEntriesInUse - 1;
+    ++indx->aIndex[i].dwDuration;
 
     /* update the standard index. Calculate the file position of
        the new frame. */
 
     GetDirectoryEntry(frame_chunk, type, name, length, offset, parent);
-    i = ix00.nEntriesInUse++;
-    ix00.aIndex[i].dwOffset = offset - ix00.qwBaseOffset;
-    ix00.aIndex[i].dwSize = length;
+    i = ix00->nEntriesInUse++;
+    ix00->aIndex[i].dwOffset = offset - ix00->qwBaseOffset;
+    ix00->aIndex[i].dwSize = length;
 }
 
 
@@ -507,25 +610,19 @@ void AVIFile::WriteRIFF()
 {}
 
 
-
 void AVIFile::WriteFrame(const Frame &frame)
 {}
 
 
-
 AVI1File::AVI1File() : AVIFile(), strl1_list( -1), strh1_chunk( -1), strf1_chunk( -1), odml_list( -1), dmlh_chunk( -1)
 {}
 
 
 
-AVI1File::AVI1File(const char *s) : AVIFile(s), strl1_list( -1), strh1_chunk( -1), strf1_chunk( -1), odml_list( -1), dmlh_chunk( -1)
-{}
-
-
-
 AVI1File::~AVI1File()
 {}
 
+
 /* Initialize the AVI structure to its initial state, either for PAL
    or NTSC format */
 
@@ -575,12 +672,12 @@ void AVI1File::Init(int format, int sampleFrequency)
         dvinfo.dwDVReserved[1] = 0;
         break;
 
-    default:                 /* no default allowed */
+    default: /* no default allowed */
         assert(0);
         break;
     }
 
-    indx.dwChunkId = make_fourcc("00__");
+    indx->dwChunkId = make_fourcc("00__");
 
     /* Initialize the strh chunk */
 
@@ -655,6 +752,7 @@ void AVI1File::Init(int format, int sampleFrequency)
     ix00_chunk = -1;
 }
 
+
 /* Write a DV video frame. This is somewhat complex... */
 
 void AVI1File::WriteFrame(const Frame &frame)
@@ -697,24 +795,27 @@ void AVI1File::WriteFrame(const Frame &frame)
        size. If so, start a new (extended) RIFF. The only allowed
        item in the new RIFF chunk is a movi list (with video
        frames and indexes as usual). */
+#if 0
 
-    //        GetDirectoryEntry(riff_list, type, name, length, offset, parent);
-    //        if (length > 0x3f000000) {
-    //                riff_list = AddDirectoryEntry(make_fourcc("RIFF"), make_fourcc("AVIX"), RIFF_LISTSIZE, file_list);
-    //                movi_list = AddDirectoryEntry(make_fourcc("LIST"), make_fourcc("movi"), RIFF_LISTSIZE, riff_list);
-    //        }
+    GetDirectoryEntry(riff_list, type, name, length, offset, parent);
+    if (length > 0x3f000000) {
+        riff_list = AddDirectoryEntry
+                    (make_fourcc("RIFF"), make_fourcc("AVIX"), RIFF_LISTSIZE, file_list);
+        movi_list = AddDirectoryEntry
+                    (make_fourcc("LIST"), make_fourcc("movi"), RIFF_LISTSIZE, riff_list);
+    }
+#endif
 }
 
 
-
 void AVI1File::WriteRIFF()
 {
     WriteChunk(avih_chunk, (void*)&mainHdr);
     WriteChunk(strh1_chunk, (void*)&stream1Hdr);
     WriteChunk(strf1_chunk, (void*)&dvinfo);
-    WriteChunk(indx_chunk, (void*)&indx);
+    WriteChunk(indx_chunk, (void*)indx);
     WriteChunk(dmlh_chunk, (void*)&dmlh);
-    WriteChunk(ix00_chunk, (void*)&ix00);
+    WriteChunk(ix00_chunk, (void*)ix00);
 
     RIFFFile::WriteRIFF();
 }
@@ -724,15 +825,10 @@ AVI2File::AVI2File() : AVIFile(), strl1_list( -1), strh1_chunk( -1), strf1_chunk
 {}
 
 
-
-AVI2File::AVI2File(const char *s) : AVIFile(s), strl1_list( -1), strh1_chunk( -1), strf1_chunk( -1), strl2_list( -1), strh2_chunk( -1), strf2_chunk( -1)
-{}
-
-
-
 AVI2File::~AVI2File()
 {}
 
+
 /* Initialize the AVI structure to its initial state, either for PAL
    or NTSC format */
 
@@ -885,19 +981,20 @@ void AVI2File::Init(int format, int sampleFrequency)
     waveformatex.nBlockAlign = 4;
     waveformatex.wBitsPerSample = 16;
 
-    for (int i = 0; i < 4000; ++i) {
-        idx1.aIndex[i].dwChunkId = 0;
-        idx1.aIndex[i].dwFlags = 0;
-        idx1.aIndex[i].dwOffset = 0;
-        idx1.aIndex[i].dwSize = 0;
+    for (int i = 0; i < 8000; ++i) {
+        idx1->aIndex[i].dwChunkId = 0;
+        idx1->aIndex[i].dwFlags = 0;
+        idx1->aIndex[i].dwOffset = 0;
+        idx1->aIndex[i].dwSize = 0;
     }
-    idx1.nEntriesInUse = 0;
+    idx1->nEntriesInUse = 0;
 
     //        for (int i = 0; i < 64000; ++i)
     //                idx1[i] = 0;
     //        idx1_entries = 0;
 
-    file_list = AddDirectoryEntry(make_fourcc("FILE"), make_fourcc("FILE"), 0, RIFF_NO_PARENT);
+    file_list = AddDirectoryEntry
+                (make_fourcc("FILE"), make_fourcc("FILE"), 0, RIFF_NO_PARENT);
 
     /* Create a basic directory structure. Only chunks defined from here on will be written to the AVI file. */
 
@@ -921,12 +1018,13 @@ void AVI2File::WriteRIFF()
     WriteChunk(strf1_chunk, (void*)&bitmapinfo);
     WriteChunk(strh2_chunk, (void*)&stream2Hdr);
     WriteChunk(strf2_chunk, (void*)&waveformatex);
-    int idx1_chunk = AddDirectoryEntry(make_fourcc("idx1"), 0, idx1.nEntriesInUse * 16, riff_list);
-    WriteChunk(idx1_chunk, (void*)&idx1);
+    int idx1_chunk = AddDirectoryEntry(make_fourcc("idx1"), 0, idx1->nEntriesInUse * 16, riff_list);
+    WriteChunk(idx1_chunk, (void*)idx1);
 
     RIFFFile::WriteRIFF();
 }
 
+
 /** Write a DV video frame
  
     \param frame the frame to write
@@ -948,22 +1046,22 @@ void AVI2File::WriteFrame(const Frame &frame)
         audio_chunk = AddDirectoryEntry(make_fourcc("01wb"), 0, audio_size, movi_list);
         stream2Hdr.dwLength += audio_size;
         WriteChunk(audio_chunk, soundbuf);
-        idx1.aIndex[idx1.nEntriesInUse].dwChunkId = make_fourcc("01wb");
-        idx1.aIndex[idx1.nEntriesInUse].dwFlags = 0;
-        idx1.aIndex[idx1.nEntriesInUse].dwOffset = GetDirectoryEntry(audio_chunk).offset - RIFF_HEADERSIZE;
-        idx1.aIndex[idx1.nEntriesInUse].dwSize = audio_size;
-        idx1.nEntriesInUse++;
+        idx1->aIndex[idx1->nEntriesInUse].dwChunkId = make_fourcc("01wb");
+        idx1->aIndex[idx1->nEntriesInUse].dwFlags = 0;
+        idx1->aIndex[idx1->nEntriesInUse].dwOffset = GetDirectoryEntry(audio_chunk).offset - GetDirectoryEntry(movi_list).offset - RIFF_HEADERSIZE;
+        idx1->aIndex[idx1->nEntriesInUse].dwSize = audio_size;
+        idx1->nEntriesInUse++;
     }
 
     /* Write video data */
 
     frame_chunk = AddDirectoryEntry(make_fourcc("00dc"), 0, frame.GetFrameSize(), movi_list);
     WriteChunk(frame_chunk, frame.data);
-    idx1.aIndex[idx1.nEntriesInUse].dwChunkId = make_fourcc("00dc");
-    idx1.aIndex[idx1.nEntriesInUse].dwFlags = 0x10;
-    idx1.aIndex[idx1.nEntriesInUse].dwOffset = GetDirectoryEntry(frame_chunk).offset - RIFF_HEADERSIZE;
-    idx1.aIndex[idx1.nEntriesInUse].dwSize = frame.GetFrameSize();
-    idx1.nEntriesInUse++;
+    idx1->aIndex[idx1->nEntriesInUse].dwChunkId = make_fourcc("00dc");
+    idx1->aIndex[idx1->nEntriesInUse].dwFlags = 0x10;
+    idx1->aIndex[idx1->nEntriesInUse].dwOffset = GetDirectoryEntry(frame_chunk).offset - GetDirectoryEntry(movi_list).offset - RIFF_HEADERSIZE;
+    idx1->aIndex[idx1->nEntriesInUse].dwSize = frame.GetFrameSize();
+    idx1->nEntriesInUse++;
 
     /* update some variables with the new frame count. */
 
