@@ -25,12 +25,44 @@
     \file frame.cc
 */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+// C includes
+
+#include <pthread.h>
+
+// C++ includes
+
+#include <string>
+#include <iostream>
+#include <strstream>
+#include <iomanip>
+
+using std::strstream;
+using std::setw;
+using std::setfill;
+
+// local includes
+#include "libdv.h"
 #include "frame.h"
-#include "dv_codec.h"
-#include <stdio.h> 
-#include <string.h> 
+#include "kino_common.h"
+
+// this is ugly.
+#ifdef WITH_LIBDV
+#include "preferences.h"
+dv_decoder_t* Frame::decoder;
+#endif
+
+extern Preferences prefs;
+
+VideoInfo::VideoInfo(): width(0), height(0), isPAL(false)
+{}
+
 
 bool Frame::maps_initialized = false;
+bool Frame::header_seen = false;
 int Frame::palmap_ch1[2000];
 int Frame::palmap_ch2[2000];
 int Frame::palmap_2ch1[2000];
@@ -48,22 +80,26 @@ short Frame::compmap[4096];
     All Frame objects share a set of lookup maps,
     which are initalized once (we are using a variant of the Singleton pattern). 
  
-    \ingroup frame 
 */
 
-
 Frame::Frame() : bytesInFrame(0)
 {
     memset(data, 0, 144000);
 
     if (maps_initialized == false) {
 
+#ifdef WITH_LIBDV
+        decoder = LibDV::DV_decoder_new();
+        LibDV::DV_init();
+#endif
+
         for (int n = 0; n < 1944; ++n) {
             int sequence1 = ((n / 3) + 2 * (n % 3)) % 6;
             int sequence2 = sequence1 + 6;
             int block = 3 * (n % 3) + ((n % 54) / 18);
 
-            block = 6 + block * 16; {
+            block = 6 + block * 16;
+            {
                 register int byte = 8 + 2 * (n / 54);
                 palmap_ch1[n] = sequence1 * 150 * 80 + block * 80 + byte;
                 palmap_ch2[n] = sequence2 * 150 * 80 + block * 80 + byte;
@@ -77,7 +113,8 @@ Frame::Frame() : bytesInFrame(0)
             int sequence2 = sequence1 + 5;
             int block = 3 * (n % 3) + ((n % 45) / 15);
 
-            block = 6 + block * 16; {
+            block = 6 + block * 16;
+            {
                 register int byte = 8 + 2 * (n / 45);
                 ntscmap_ch1[n] = sequence1 * 150 * 80 + block * 80 + byte;
                 ntscmap_ch2[n] = sequence2 * 150 * 80 + block * 80 + byte;
@@ -86,16 +123,43 @@ Frame::Frame() : bytesInFrame(0)
                 ntscmap_2ch2[n] = sequence2 * 150 * 80 + block * 80 + byte;
             }
         }
-        for (int y = 0x700; y <= 0x7ff; ++y) compmap[y] = (y - 0x600) << 6;
-        for (int y = 0x600; y <= 0x6ff; ++y) compmap[y] = (y - 0x500) << 5;
-        for (int y = 0x500; y <= 0x5ff; ++y) compmap[y] = (y - 0x400) << 4;
-        for (int y = 0x400; y <= 0x4ff; ++y) compmap[y] = (y - 0x300) << 3;
-        for (int y = 0x300; y <= 0x3ff; ++y) compmap[y] = (y - 0x200) << 2;
-        for (int y = 0x200; y <= 0x2ff; ++y) compmap[y] = (y - 0x100) << 1;
-        for (int y = 0x000; y <= 0x1ff; ++y) compmap[y] = y;
-        for (int y = 0x800; y <= 0xfff; ++y) compmap[y] = -1 - compmap[0xfff - y];
+        for (int y = 0x700; y <= 0x7ff; ++y)
+            compmap[y] = (y - 0x600) << 6;
+        for (int y = 0x600; y <= 0x6ff; ++y)
+            compmap[y] = (y - 0x500) << 5;
+        for (int y = 0x500; y <= 0x5ff; ++y)
+            compmap[y] = (y - 0x400) << 4;
+        for (int y = 0x400; y <= 0x4ff; ++y)
+            compmap[y] = (y - 0x300) << 3;
+        for (int y = 0x300; y <= 0x3ff; ++y)
+            compmap[y] = (y - 0x200) << 2;
+        for (int y = 0x200; y <= 0x2ff; ++y)
+            compmap[y] = (y - 0x100) << 1;
+        for (int y = 0x000; y <= 0x1ff; ++y)
+            compmap[y] = y;
+        for (int y = 0x800; y <= 0xfff; ++y)
+            compmap[y] = -1 - compmap[0xfff - y];
         maps_initialized = true;
     }
+#ifdef WITH_LIBDV
+    switch ((int) Preferences::getInstance().displayQuality) {
+    case 5:
+        decoder->quality = DV_QUALITY_FASTEST;
+        break;
+    case 4:
+        decoder->quality = DV_QUALITY_AC_1;
+        break;
+    case 3:
+        decoder->quality = DV_QUALITY_COLOR | DV_QUALITY_DC;
+        break;
+    case 2:
+        decoder->quality = DV_QUALITY_COLOR | DV_QUALITY_AC_1;
+        break;
+    default:
+        decoder->quality = DV_QUALITY_BEST;
+        break;
+    }
+#endif
 }
 
 
@@ -107,7 +171,6 @@ Frame::~Frame()
  
     This function returns a SSYB packet from the subcode data section.
  
-    \ingroup frame
     \param packNum the VAUX package id to return
     \param pack a reference to the variable where the result is stored
     \return true for success, false if no pack could be found */
@@ -161,7 +224,6 @@ bool Frame::GetSSYBPack(int packNum, Pack &pack) const
     up, but I choose to walk through all data as with the other
     GetXXXX routines.
  
-    \ingroup frame
     \param packNum the VAUX package id to return
     \param pack a reference to the variable where the result is stored
     \return true for success, false if no pack could be found */
@@ -214,7 +276,6 @@ bool Frame::GetVAUXPack(int packNum, Pack &pack) const
     through all DIF blocks although AAUX packets are only allowed in
     certain defined DIF blocks.
  
-    \ingroup frame
     \param packNum the AAUX package id to return
     \param pack a reference to the variable where the result is stored
     \return true for success, false if no pack could be found */
@@ -261,7 +322,6 @@ bool Frame::GetAAUXPack(int packNum, Pack &pack) const
  
     This code courtesy of Andy (http://www.videox.net/) 
  
-    \ingroup frame
     \param recDate the time and date of recording of this frame
     \return true for success, false if no date or time information could be found */
 
@@ -314,13 +374,34 @@ bool Frame::GetRecordingDate(struct tm &recDate) const
 }
 
 
+string Frame::GetRecordingDate(void) const
+{
+    struct tm date;
+    strstream sb;
+    string recDate;
+
+    if (GetRecordingDate(date) == true) {
+        sb << setfill('0')
+        << setw(4) << date.tm_year + 1900 << '.'
+        << setw(2) << date.tm_mon + 1 << '.'
+        << setw(2) << date.tm_mday << '_'
+        << setw(2) << date.tm_hour << '-'
+        << setw(2) << date.tm_min << '-'
+        << setw(2) << date.tm_sec;
+        sb >> recDate;
+    } else {
+        recDate = "0000.00.00_00-00-00";
+    }
+    return recDate;
+}
+
+
 /** gets the timecode information of this frame
  
     Returns a string with the timecode of this frame. The timecode is
     the relative location of this frame on the tape, and is defined
     by hour, minute, second and frame (within the last second).
      
-    \ingroup frame
     \param timeCode the TimeCode struct
     \return true for success, false if no timecode information could be found */
 
@@ -359,6 +440,7 @@ bool Frame::GetAudioInfo(AudioInfo &info) const
     int flag;
     Pack pack50;
 
+
     /* Check whether this frame has a valid AAUX source packet
        (header == 0x50). If so, get the audio samples count. If not,
        skip this audio data. */
@@ -412,11 +494,19 @@ bool Frame::GetAudioInfo(AudioInfo &info) const
 }
 
 
+bool Frame::GetVideoInfo(VideoInfo &info) const
+{
+    GetTimeCode(info.timeCode);
+    GetRecordingDate(info.recDate);
+    info.isPAL = IsPAL();
+    return true;
+}
+
+
 /** gets the size of the frame
  
     Depending on the type (PAL or NTSC) of the frame, the length of the frame is returned 
  
-    \ingroup frame
     \return the length of the frame in Bytes */
 
 int Frame::GetFrameSize(void) const
@@ -428,7 +518,6 @@ int Frame::GetFrameSize(void) const
 /** checks whether the frame is in PAL or NTSC format
  
     \todo function can't handle "empty" frame
-    \ingroup frame
     \return true for PAL frame, false for a NTSC frame
 */
 
@@ -439,13 +528,31 @@ bool Frame::IsPAL(void) const
     return dsf == 0 ? false : true;
 }
 
+/** checks whether the frame is in PAL or NTSC format
+ 
+    \todo function can't handle "empty" frame
+    \return true for PAL frame, false for a NTSC frame
+*/
+
+bool Frame::IsPALfromHeader (void) const
+{
+    unsigned char dsf = data[3] & 0x80;
+    bool pal = (dsf == 0) ? false : true;
+#ifdef WITH_LIBDV
+
+    if (!pal && header_seen) {
+        pal = (LibDV::DV_system_50_fields (decoder) == 1) ? true : pal;
+    }
+#endif
+    return pal;
+}
+
 
 /** checks whether this frame is the first in a new recording
  
     To determine this, the function looks at the recStartPoint bit in
     AAUX pack 51.
  
-    \ingroup frame
     \return true for PAL frame, false for a NTSC frame */
 
 bool Frame::IsNewRecording() const
@@ -498,33 +605,23 @@ int Frame::ExtractAudio(void *sound) const
 
         switch (info.frequency) {
         case 32000:
-            // This is 4 channel audio
 
+            /* This is 4 channel audio */
 
             if (IsPAL()) {
                 short* p = (short*)sound;
                 for (int n = 0; n < info.samples; ++n) {
-                    register int r
-                    = ((unsigned char*)data)[palmap_2ch1[n] + 1]; // LSB
-                    *p++ = compmap[(((unsigned char*)data)[palmap_2ch1[n]] << 4)
-                                   + (r >> 4)];   // MSB
-                    *p++ = compmap[(((unsigned char*)data)[palmap_2ch1[n] + 1] << 4)
-                                   + (r & 0x0f)]; // MSB
+                    register int r = ((unsigned char*)data)[palmap_2ch1[n] + 1]; // LSB
+                    *p++ = compmap[(((unsigned char*)data)[palmap_2ch1[n]] << 4) + (r >> 4)];   // MSB
+                    *p++ = compmap[(((unsigned char*)data)[palmap_2ch1[n] + 1] << 4) + (r & 0x0f)];
                 }
-
-
             } else {
                 short* p = (short*)sound;
                 for (int n = 0; n < info.samples; ++n) {
-                    register int r
-                    = ((unsigned char*)data)[ntscmap_2ch1[n] + 1]; // LSB
-                    *p++ = compmap[(((unsigned char*)data)[ntscmap_2ch1[n]] << 4)
-                                   + (r >> 4)];   // MSB
-                    *p++ = compmap[(((unsigned char*)data)[ntscmap_2ch1[n] + 1] << 4)
-                                   + (r & 0x0f)]; // MSB
+                    register int r = ((unsigned char*)data)[ntscmap_2ch1[n] + 1]; // LSB
+                    *p++ = compmap[(((unsigned char*)data)[ntscmap_2ch1[n]] << 4) + (r >> 4)];   // MSB
+                    *p++ = compmap[(((unsigned char*)data)[ntscmap_2ch1[n] + 1] << 4) + (r & 0x0f)];
                 }
-
-
             }
             break;
 
@@ -556,14 +653,80 @@ int Frame::ExtractAudio(void *sound) const
             info.samples = 0;
         }
     }
+	else {
+        info.samples = 0;
+	}
 
     return info.samples * 4;
 }
 
+#ifdef WITH_LIBDV
 
-int Frame::ExtractRGB(void *rgb) const
+void Frame::ExtractHeader(void)
 {
+    if (header_seen == false) {
+        LibDV::DV_parse_header(decoder, data);
+        header_seen = true;
+    }
+}
 
-    dv_codec_decode(rgb, data);
+
+int Frame::ExtractRGB(void *rgb)
+{
+    guchar *pixels[3];
+    gint pitches[3];
+
+    pixels[0] = (guchar*)rgb;
+    pixels[1] = NULL;
+    pixels[2] = NULL;
+
+    pitches[0] = 720 * 3;
+    pitches[1] = 0;
+    pitches[2] = 0;
+
+    LibDV::DV_decode_full_frame(decoder, data, e_dv_color_rgb, pixels, pitches);
     return 0;
 }
+
+
+int Frame::ExtractBGR0(void *rgb)
+{
+    guchar *pixels[3];
+    gint pitches[3];
+
+    pixels[0] = (guchar*)rgb;
+    pixels[1] = NULL;
+    pixels[2] = NULL;
+
+    pitches[0] = 720 * 4;
+    pitches[1] = 0;
+    pitches[2] = 0;
+
+    LibDV::DV_decode_full_frame(decoder, data, e_dv_color_bgr0, pixels, pitches);
+    return 0;
+}
+
+
+int Frame::ExtractYUV(void *yuv)
+{
+    guchar *pixels[3];
+    gint pitches[3];
+
+    pixels[0] = (guchar*)yuv;
+    pitches[0] = decoder->width * 2;
+
+    LibDV::DV_decode_full_frame(decoder, data, e_dv_color_yuv, pixels, pitches);
+    return 0;
+}
+
+/** Get the frame aspect ratio.
+
+	Indicates whether frame aspect ration is normal (4:3) or wide (16:9).
+
+    /return true if the frame is wide (16:9), false if unknown or normal.
+*/
+bool Frame::IsWide( void ) const
+{
+	return LibDV::DV_format_wide(decoder) > 0;
+}
+#endif
